import 'package:fluro/fluro.dart';
import 'package:flutterhole/bloc/api/versions.dart';
import 'package:flutterhole/model/api/blacklist.dart';
import 'package:flutterhole/model/api/client_names.dart';
import 'package:flutterhole/model/api/clients_over_time.dart';
import 'package:flutterhole/model/api/forward_destinations.dart';
import 'package:flutterhole/model/api/queries_over_time.dart';
import 'package:flutterhole/model/api/query.dart';
import 'package:flutterhole/model/api/status.dart';
import 'package:flutterhole/model/api/summary.dart';
import 'package:flutterhole/model/api/top_items.dart';
import 'package:flutterhole/model/api/top_sources.dart';
import 'package:flutterhole/model/api/versions.dart';
import 'package:flutterhole/model/api/whitelist.dart';
import 'package:flutterhole/model/pihole.dart';
import 'package:flutterhole/service/memory_tree.dart';
import 'package:flutterhole/service/pihole_client.dart';
import 'package:flutterhole/service/secure_store.dart';
import 'package:mockito/mockito.dart';

final mockPiholes = [
  Pihole(),
  Pihole(title: 'second', host: '2.com'),
  Pihole(title: 'third', host: '3.net'),
];

class MockPiholeClient extends Mock implements PiholeClient {}

class MockVersionsRepository extends Mock implements VersionsRepository {}

class MockRouter extends Mock implements Router {}

class MockMemoryTree extends Mock implements MemoryTree {}

class MockSecureStore extends Mock implements SecureStore {
  @override
  Pihole active;

  @override
  Map<String, Pihole> piholes;
}

final mockStatusEnabled = Status(enabled: true);
final mockStatusDisabled = Status(enabled: false);

final mockWhitelist = Whitelist(['a.com', 'b.org', 'c.net']);
final mockBlacklist = Blacklist(exact: [
  BlacklistItem.exact(entry: 'exact.com'),
  BlacklistItem.exact(entry: 'pi-hole.net'),
], wildcard: [
  BlacklistItem.wildcard(entry: '${wildcardPrefix}wildcard.com$wildcardSuffix'),
  BlacklistItem.regex(entry: 'regex'),
]);

final mockSummary = Summary(
    domainsBeingBlocked: 0,
    dnsQueriesToday: 1,
    adsBlockedToday: 2,
    adsPercentageToday: 2.345,
    uniqueDomains: 3,
    queriesForwarded: 4,
    queriesCached: 5,
    clientsEverSeen: 6,
    uniqueClients: 7,
    dnsQueriesAllTypes: 8,
    replyNodata: 9,
    replyNxdomain: 10,
    replyCname: 11,
    replyIp: 12,
    privacyLevel: 13,
    status: 'enabled',
    gravityLastUpdated: GravityLastUpdated(
        fileExists: true,
        absolute: 14,
        relative: Relative(days: '15', hours: '16', minutes: '17')));

final TopSources mockTopSources = TopSources({
  '10.0.1.1': 55,
  '10.0.1.2': 42,
  'windows|10.0.1.3': 33,
  'osx|10.0.1.4': 24,
});

final Versions mockVersions = Versions(
    coreUpdate: true,
    webUpdate: false,
    ftlUpdate: false,
    coreCurrent: 'v1.2.3',
    webCurrent: 'v1.2.4',
    ftlCurrent: 'v1.2.5',
    coreLatest: "",
    webLatest: "",
    ftlLatest: "",
    coreBranch: 'master',
    webBranch: 'master',
    ftlBranch: 'master');

final QueryTypes mockQueryTypes = QueryTypes({
  "A (IPv4)": 58.46,
  "AAAA (IPv6)": 10.12,
  "ANY": 10.50,
  "SRV": 0.45,
  "SOA": 9.50,
  "PTR": 2.97,
  "TXT": 8.0,
});

final TopItems mockTopItems = TopItems(
  topQueries: {
    "a.com": 1,
    "b.org": 2,
    "c.net": 3,
  },
  topAds: {
    "ad1.com": 1,
    "ad2.org": 2,
    "ad3.net": 3,
  },
);

final ForwardDestinations mockForwardDestinations = ForwardDestinations({
  "blocklist|blocklist": 9.7,
  "cache|cache": 14.37,
  "dns.google|8.8.4.4": 40.41,
  "dns.google|8.8.8.8": 35.53
});

final ClientNames mockClientNames = ClientNames([
  Client(name: 'localhost', ip: '127.0.0.1'),
  Client(name: 'second', ip: '1.2.3.4'),
  Client(name: '', ip: '10.0.1.1'),
]);

final List<Query> mockQueries = [
  Query(
      time: DateTime.fromMillisecondsSinceEpoch(1563995000000),
      queryType: QueryType.A,
      entry: 'test.com',
      client: 'localhost',
      queryStatus: QueryStatus.Cached,
      dnsSecStatus: DnsSecStatus.Secure),
  Query(
      time: DateTime(1),
      queryType: QueryType.PTR,
      entry: 'example.org',
      client: 'remotehost',
      queryStatus: QueryStatus.Unknown,
      dnsSecStatus: DnsSecStatus.Abandoned),
];

final mockQueriesOverTime = QueriesOverTime(adsOverTime: {
  '1563991500000': 1,
  '1563992100000': 0,
  '1563992700000': 1,
  '1563993300000': 2,
  '1563993900000': 0,
  '1563994500000': 0,
  '1563995100000': 0,
  '1563995700000': 0,
  '1563996300000': 0,
  '1563996900000': 0,
  '1563997500000': 2,
  '1563998100000': 9,
  '1563998700000': 17,
  '1563999300000': 1,
  '1563999900000': 17,
  '1564000500000': 6,
  '1564001100000': 13,
  '1564001700000': 6,
  '1564002300000': 23,
  '1564002900000': 16,
  '1564003500000': 11,
  '1564004100000': 11,
  '1564004700000': 6,
  '1564005300000': 2,
  '1564005900000': 0,
  '1564006500000': 0,
  '1564007100000': 0,
  '1564007700000': 0,
  '1564008300000': 3,
  '1564008900000': 20,
  '1564009500000': 20,
  '1564010100000': 27,
  '1564010700000': 25,
  '1564011300000': 46,
  '1564011900000': 33,
  '1564012500000': 16,
  '1564013100000': 10,
  '1564013700000': 14,
  '1564014300000': 15,
  '1564014900000': 4,
  '1564015500000': 4,
  '1564016100000': 3,
  '1564016700000': 5,
  '1564017300000': 0,
  '1564017900000': 0,
  '1564018500000': 13,
  '1564019100000': 1,
  '1564019700000': 0,
  '1564020300000': 1,
  '1564020900000': 0,
  '1564021500000': 0,
  '1564022100000': 0,
  '1564022700000': 0,
  '1564023300000': 0,
  '1564023900000': 0,
  '1564024500000': 0,
  '1564025100000': 0,
  '1564025700000': 0,
  '1564026300000': 0,
  '1564026900000': 6,
  '1564027500000': 0,
  '1564028100000': 1,
  '1564028700000': 0,
  '1564029300000': 0,
  '1564029900000': 1,
  '1564030500000': 2,
  '1564031100000': 2,
  '1564031700000': 3,
  '1564032300000': 3,
  '1564032900000': 7,
  '1564033500000': 2,
  '1564034100000': 4,
  '1564034700000': 2,
  '1564035300000': 3,
  '1564035900000': 3,
  '1564036500000': 3,
  '1564037100000': 1,
  '1564037700000': 0,
  '1564038300000': 4,
  '1564038900000': 0,
  '1564039500000': 0,
  '1564040100000': 0,
  '1564040700000': 0,
  '1564041300000': 0,
  '1564041900000': 25,
  '1564042500000': 36,
  '1564043100000': 14,
  '1564043700000': 1,
  '1564044300000': 18,
  '1564044900000': 11,
  '1564045500000': 19,
  '1564046100000': 19,
  '1564046700000': 16,
  '1564047300000': 16,
  '1564047900000': 13,
  '1564048500000': 9,
  '1564049100000': 21,
  '1564049700000': 18,
  '1564050300000': 10,
  '1564050900000': 15,
  '1564051500000': 9,
  '1564052100000': 15,
  '1564052700000': 15,
  '1564053300000': 15,
  '1564053900000': 13,
  '1564054500000': 22,
  '1564055100000': 10,
  '1564055700000': 7,
  '1564056300000': 10,
  '1564056900000': 15,
  '1564057500000': 20,
  '1564058100000': 12,
  '1564058700000': 12,
  '1564059300000': 5,
  '1564059900000': 5,
  '1564060500000': 12,
  '1564061100000': 4,
  '1564061700000': 15,
  '1564062300000': 8,
  '1564062900000': 11,
  '1564063500000': 12,
  '1564064100000': 7,
  '1564064700000': 5,
  '1564065300000': 9,
  '1564065900000': 6,
  '1564066500000': 7,
  '1564067100000': 5,
  '1564067700000': 6,
  '1564068300000': 10,
  '1564068900000': 24,
  '1564069500000': 7,
  '1564070100000': 10,
  '1564070700000': 8,
  '1564071300000': 17,
  '1564071900000': 11,
  '1564072500000': 6,
  '1564073100000': 10,
  '1564073700000': 7,
  '1564074300000': 7,
  '1564074900000': 13
}, clientsOverTime: {
  '1563991500000': 122,
  '1563992100000': 97,
  '1563992700000': 75,
  '1563993300000': 79,
  '1563993900000': 25,
  '1563994500000': 13,
  '1563995100000': 14,
  '1563995700000': 25,
  '1563996300000': 10,
  '1563996900000': 83,
  '1563997500000': 74,
  '1563998100000': 68,
  '1563998700000': 90,
  '1563999300000': 33,
  '1563999900000': 83,
  '1564000500000': 32,
  '1564001100000': 53,
  '1564001700000': 54,
  '1564002300000': 87,
  '1564002900000': 72,
  '1564003500000': 43,
  '1564004100000': 31,
  '1564004700000': 41,
  '1564005300000': 20,
  '1564005900000': 43,
  '1564006500000': 63,
  '1564007100000': 44,
  '1564007700000': 21,
  '1564008300000': 18,
  '1564008900000': 89,
  '1564009500000': 52,
  '1564010100000': 59,
  '1564010700000': 50,
  '1564011300000': 74,
  '1564011900000': 69,
  '1564012500000': 41,
  '1564013100000': 35,
  '1564013700000': 38,
  '1564014300000': 76,
  '1564014900000': 49,
  '1564015500000': 42,
  '1564016100000': 33,
  '1564016700000': 28,
  '1564017300000': 19,
  '1564017900000': 11,
  '1564018500000': 52,
  '1564019100000': 12,
  '1564019700000': 21,
  '1564020300000': 19,
  '1564020900000': 14,
  '1564021500000': 0,
  '1564022100000': 7,
  '1564022700000': 1,
  '1564023300000': 0,
  '1564023900000': 4,
  '1564024500000': 6,
  '1564025100000': 1,
  '1564025700000': 2,
  '1564026300000': 1,
  '1564026900000': 26,
  '1564027500000': 4,
  '1564028100000': 5,
  '1564028700000': 0,
  '1564029300000': 2,
  '1564029900000': 10,
  '1564030500000': 4,
  '1564031100000': 11,
  '1564031700000': 7,
  '1564032300000': 5,
  '1564032900000': 11,
  '1564033500000': 9,
  '1564034100000': 10,
  '1564034700000': 15,
  '1564035300000': 16,
  '1564035900000': 5,
  '1564036500000': 13,
  '1564037100000': 1,
  '1564037700000': 5,
  '1564038300000': 10,
  '1564038900000': 4,
  '1564039500000': 0,
  '1564040100000': 1,
  '1564040700000': 0,
  '1564041300000': 3,
  '1564041900000': 71,
  '1564042500000': 125,
  '1564043100000': 77,
  '1564043700000': 16,
  '1564044300000': 136,
  '1564044900000': 65,
  '1564045500000': 69,
  '1564046100000': 58,
  '1564046700000': 58,
  '1564047300000': 36,
  '1564047900000': 39,
  '1564048500000': 17,
  '1564049100000': 44,
  '1564049700000': 51,
  '1564050300000': 43,
  '1564050900000': 31,
  '1564051500000': 25,
  '1564052100000': 30,
  '1564052700000': 34,
  '1564053300000': 42,
  '1564053900000': 25,
  '1564054500000': 42,
  '1564055100000': 36,
  '1564055700000': 27,
  '1564056300000': 43,
  '1564056900000': 45,
  '1564057500000': 58,
  '1564058100000': 41,
  '1564058700000': 36,
  '1564059300000': 21,
  '1564059900000': 29,
  '1564060500000': 50,
  '1564061100000': 13,
  '1564061700000': 28,
  '1564062300000': 53,
  '1564062900000': 38,
  '1564063500000': 46,
  '1564064100000': 34,
  '1564064700000': 30,
  '1564065300000': 27,
  '1564065900000': 25,
  '1564066500000': 25,
  '1564067100000': 31,
  '1564067700000': 25,
  '1564068300000': 34,
  '1564068900000': 70,
  '1564069500000': 26,
  '1564070100000': 19,
  '1564070700000': 30,
  '1564071300000': 59,
  '1564071900000': 84,
  '1564072500000': 46,
  '1564073100000': 54,
  '1564073700000': 40,
  '1564074300000': 63,
  '1564074900000': 69
});

final mockClientsOverTime = ClientsOverTime(overTime: {
  "1564783500": [0, 23, 0, 6, 0, 0, 0, 0, 0, 0, 0],
  "1564784100": [0, 23, 0, 2, 0, 0, 0, 0, 0, 0, 0],
  "1564784700": [0, 89, 1, 3, 0, 4, 0, 0, 0, 0, 0],
  "1564785300": [0, 50, 0, 5, 0, 0, 0, 0, 0, 0, 0],
  "1564785900": [0, 40, 0, 4, 0, 0, 0, 0, 0, 0, 0],
  "1564786500": [0, 46, 1, 7, 0, 0, 0, 0, 0, 0, 0],
  "1564787100": [6, 67, 0, 10, 0, 0, 0, 0, 0, 0, 0],
  "1564787700": [0, 127, 0, 6, 1, 0, 0, 0, 0, 0, 0],
  "1564788300": [0, 52, 1, 9, 0, 4, 0, 0, 0, 0, 0],
  "1564788900": [0, 6, 0, 40, 0, 0, 0, 0, 0, 0, 0],
  "1564789500": [0, 0, 0, 95, 0, 0, 0, 0, 2, 0, 0],
  "1564790100": [0, 0, 1, 7, 0, 0, 0, 0, 0, 0, 0],
  "1564790700": [6, 0, 0, 34, 0, 0, 0, 0, 0, 0, 0],
  "1564791300": [0, 0, 0, 9, 0, 0, 0, 0, 0, 0, 0],
  "1564791900": [0, 0, 1, 17, 0, 4, 0, 0, 0, 0, 0],
  "1564792500": [0, 0, 0, 1, 4, 0, 0, 0, 0, 0, 0],
  "1564793100": [0, 0, 0, 7, 0, 0, 0, 0, 0, 0, 0],
  "1564793700": [0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0],
  "1564794300": [6, 0, 0, 4, 0, 0, 0, 0, 0, 0, 0],
  "1564794900": [0, 0, 0, 7, 0, 0, 0, 0, 0, 0, 0],
  "1564795500": [0, 0, 1, 0, 0, 4, 0, 0, 0, 0, 0],
  "1564796100": [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
  "1564796700": [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
  "1564797300": [0, 0, 3, 0, 0, 0, 0, 0, 0, 0, 0],
  "1564797900": [6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
  "1564798500": [0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0],
  "1564799100": [0, 0, 1, 0, 0, 4, 0, 0, 0, 0, 0],
  "1564799700": [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
  "1564800300": [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
  "1564800900": [0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0],
  "1564801500": [6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
  "1564802100": [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
  "1564802700": [0, 0, 1, 0, 0, 4, 0, 0, 0, 0, 0],
  "1564803300": [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
  "1564803900": [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
  "1564804500": [0, 0, 1, 19, 0, 0, 4, 0, 0, 0, 0],
  "1564805100": [6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
  "1564805700": [0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0],
  "1564806300": [0, 0, 1, 2, 0, 4, 0, 0, 0, 0, 0],
  "1564806900": [0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0],
  "1564807500": [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
  "1564808100": [0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0],
  "1564808700": [6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
  "1564809300": [0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0],
  "1564809900": [0, 0, 1, 0, 0, 4, 0, 0, 0, 0, 0],
  "1564810500": [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
  "1564811100": [0, 0, 0, 17, 1, 0, 0, 0, 0, 0, 0],
  "1564811700": [0, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0],
  "1564812300": [6, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
  "1564812900": [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
  "1564813500": [0, 0, 0, 0, 0, 4, 0, 0, 0, 0, 0],
  "1564814100": [0, 0, 1, 0, 3, 0, 0, 0, 0, 0, 0],
  "1564814700": [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
  "1564815300": [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
  "1564815900": [6, 0, 1, 4, 0, 0, 0, 0, 0, 0, 0],
  "1564816500": [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
  "1564817100": [0, 0, 0, 0, 0, 4, 0, 0, 0, 0, 0],
  "1564817700": [0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0],
  "1564818300": [0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0],
  "1564818900": [0, 0, 2, 18, 0, 0, 0, 0, 0, 0, 0],
  "1564819500": [6, 0, 1, 62, 0, 0, 0, 0, 0, 0, 0],
  "1564820100": [0, 0, 0, 82, 4, 0, 0, 0, 0, 0, 0],
  "1564820700": [0, 0, 0, 52, 0, 4, 0, 0, 3, 0, 0],
  "1564821300": [0, 0, 1, 5, 0, 0, 0, 0, 0, 0, 0],
  "1564821900": [0, 0, 0, 16, 0, 0, 0, 0, 0, 0, 0],
  "1564822500": [0, 80, 0, 12, 0, 0, 0, 0, 3, 0, 0],
  "1564823100": [6, 82, 1, 4, 0, 0, 0, 0, 0, 0, 0],
  "1564823700": [0, 26, 0, 40, 0, 0, 0, 0, 0, 0, 0],
  "1564824300": [0, 31, 0, 12, 0, 4, 0, 0, 0, 0, 0],
  "1564824900": [0, 26, 1, 6, 0, 0, 0, 0, 0, 0, 0],
  "1564825500": [0, 46, 0, 4, 0, 0, 0, 0, 0, 0, 0],
  "1564826100": [0, 39, 0, 10, 0, 0, 0, 0, 0, 0, 0],
  "1564826700": [6, 37, 1, 5, 0, 0, 0, 0, 0, 0, 0],
  "1564827300": [0, 73, 0, 12, 0, 0, 0, 0, 0, 0, 0],
  "1564827900": [0, 67, 0, 51, 0, 4, 0, 0, 0, 0, 0],
  "1564828500": [0, 35, 1, 24, 0, 0, 0, 0, 0, 0, 0],
  "1564829100": [0, 32, 0, 4, 0, 0, 0, 0, 0, 0, 0],
  "1564829700": [0, 52, 0, 7, 0, 0, 3, 0, 0, 0, 0],
  "1564830300": [6, 19, 1, 5, 0, 0, 0, 0, 0, 0, 0],
  "1564830900": [0, 27, 0, 7, 1, 0, 0, 0, 0, 0, 0],
  "1564831500": [0, 37, 0, 23, 0, 4, 0, 0, 0, 0, 0],
  "1564832100": [0, 42, 1, 9, 0, 0, 0, 0, 0, 0, 0],
  "1564832700": [0, 34, 0, 32, 0, 0, 0, 0, 0, 0, 0],
  "1564833300": [0, 46, 0, 31, 0, 0, 0, 0, 0, 0, 0],
  "1564833900": [6, 25, 1, 18, 0, 0, 0, 0, 0, 0, 0],
  "1564834500": [0, 18, 0, 0, 3, 0, 0, 0, 0, 0, 0],
  "1564835100": [0, 27, 0, 0, 0, 0, 0, 0, 0, 0, 0],
  "1564835700": [0, 13, 1, 0, 5, 4, 0, 0, 0, 0, 0],
  "1564836300": [0, 49, 0, 20, 0, 0, 0, 0, 0, 0, 0],
  "1564836900": [0, 30, 0, 43, 0, 0, 0, 0, 0, 0, 0],
  "1564837500": [6, 27, 1, 19, 0, 0, 0, 0, 0, 0, 0],
  "1564838100": [0, 20, 0, 2, 0, 0, 0, 0, 0, 0, 0],
  "1564838700": [0, 19, 0, 0, 0, 0, 0, 0, 0, 0, 0],
  "1564839300": [0, 18, 1, 0, 0, 4, 0, 0, 0, 0, 0],
  "1564839900": [0, 30, 0, 7, 0, 0, 0, 0, 0, 0, 0],
  "1564840500": [0, 33, 2, 11, 0, 0, 0, 0, 0, 0, 0],
  "1564841100": [27, 75, 1, 12, 0, 0, 0, 0, 0, 0, 0],
  "1564841700": [0, 157, 0, 9, 0, 0, 0, 0, 0, 0, 0],
  "1564842300": [0, 89, 0, 9, 0, 0, 0, 0, 0, 0, 0],
  "1564842900": [0, 13, 1, 5, 4, 4, 0, 0, 0, 0, 0],
  "1564843500": [0, 41, 0, 22, 0, 0, 0, 0, 0, 0, 0],
  "1564844100": [0, 25, 0, 69, 0, 0, 0, 0, 0, 0, 0],
  "1564844700": [6, 23, 1, 94, 0, 0, 0, 0, 0, 0, 0],
  "1564845300": [0, 79, 0, 67, 0, 0, 0, 0, 0, 0, 0],
  "1564845900": [0, 41, 0, 9, 0, 0, 0, 0, 0, 0, 0],
  "1564846500": [0, 22, 1, 5, 0, 4, 0, 0, 0, 0, 0],
  "1564847100": [0, 49, 0, 13, 0, 0, 0, 0, 0, 0, 0],
  "1564847700": [0, 32, 0, 10, 0, 0, 0, 0, 0, 0, 0],
  "1564848300": [6, 52, 1, 27, 0, 0, 0, 0, 0, 0, 0],
  "1564848900": [0, 48, 0, 19, 0, 0, 0, 0, 0, 0, 0],
  "1564849500": [0, 34, 0, 9, 0, 0, 0, 0, 0, 0, 0],
  "1564850100": [0, 28, 1, 4, 0, 4, 0, 0, 0, 0, 0],
  "1564850700": [0, 38, 0, 3, 0, 0, 0, 0, 0, 0, 0],
  "1564851300": [0, 86, 0, 3, 0, 0, 0, 0, 0, 0, 0],
  "1564851900": [6, 66, 0, 9, 0, 0, 0, 0, 0, 0, 0],
  "1564852500": [0, 34, 1, 13, 4, 0, 0, 0, 0, 0, 0],
  "1564853100": [6, 28, 0, 15, 0, 0, 0, 0, 0, 0, 0],
  "1564853700": [0, 38, 0, 8, 0, 4, 0, 0, 0, 0, 0],
  "1564854300": [0, 38, 1, 5, 0, 0, 0, 0, 0, 0, 0],
  "1564854900": [0, 106, 0, 21, 0, 0, 3, 0, 0, 0, 0],
  "1564855500": [6, 35, 0, 6, 0, 0, 0, 0, 0, 0, 0],
  "1564856100": [0, 29, 1, 8, 0, 0, 0, 0, 0, 0, 0],
  "1564856700": [0, 52, 0, 8, 0, 0, 0, 0, 0, 0, 0],
  "1564857300": [0, 105, 0, 13, 0, 4, 0, 0, 0, 0, 0],
  "1564857900": [0, 32, 1, 5, 1, 0, 0, 0, 0, 0, 0],
  "1564858500": [0, 43, 0, 21, 0, 0, 0, 0, 0, 0, 0],
  "1564859100": [6, 124, 0, 16, 2, 0, 0, 0, 0, 0, 0],
  "1564859700": [0, 37, 1, 11, 0, 0, 0, 0, 0, 0, 0],
  "1564860300": [0, 90, 0, 23, 0, 0, 0, 0, 0, 0, 0],
  "1564860900": [0, 31, 0, 13, 3, 4, 0, 0, 0, 0, 0],
  "1564861500": [0, 33, 1, 14, 0, 0, 0, 0, 0, 0, 0],
  "1564862100": [0, 39, 2, 38, 0, 0, 0, 0, 0, 0, 0],
  "1564862700": [6, 63, 0, 60, 0, 0, 0, 0, 0, 0, 0],
  "1564863300": [0, 62, 1, 27, 0, 0, 0, 0, 0, 0, 0],
  "1564863900": [0, 25, 0, 17, 0, 0, 0, 0, 0, 0, 0],
  "1564864500": [0, 33, 0, 15, 1, 4, 0, 0, 0, 0, 0],
  "1564865100": [0, 44, 1, 14, 0, 0, 0, 0, 0, 0, 0],
  "1564865700": [0, 33, 0, 21, 0, 0, 0, 0, 0, 0, 0],
  "1564866300": [6, 86, 0, 23, 0, 0, 0, 0, 0, 0, 0],
  "1564866900": [0, 86, 1, 18, 0, 0, 0, 0, 0, 0, 0],
  "1564867500": [0, 42, 0, 21, 0, 0, 0, 0, 0, 0, 0]
});
